// ignore: unused_import
import 'dart:io' show Platform;

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:rdc_flutter_sdk_2/rabbit_data.dart';
import 'package:rdc_flutter_sdk_2/src/controllers/condition_controller.dart';
import 'package:rdc_flutter_sdk_2/src/controllers/device_info_controller.dart';
import 'package:rdc_flutter_sdk_2/src/controllers/event_controller.dart';
import 'package:rdc_flutter_sdk_2/src/controllers/location_controller.dart';
import 'package:rdc_flutter_sdk_2/src/controllers/rabbit_config_controller.dart';
import 'package:rdc_flutter_sdk_2/src/controllers/user_info_controller.dart';
import 'package:rdc_flutter_sdk_2/src/controllers/version_controller.dart';
import 'package:rxdart/rxdart.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'constants/app_constants.dart';

class RabbitDataImpl implements RabbitDataInstance {
  @override
  late RabbitConfigController config;
  @override
  late UserInfoController userInfo;
  late EventController _eventController;
  late DeviceInfoController _deviceInfoController;

  @override
  late BehaviorSubject<Map<String, num>> condition;
  @override
  late BehaviorSubject<String> apiResult;
  @override
  late BehaviorSubject<String> changeDB;

  @override
  Future<RabbitDataInstance> initializeInstance(
      RabbitDataConfig configData) async {
    print('[RDC Log] [Init 1]: ${DateTime.now()}');
    config =
        Get.put(RabbitConfigController(configData), tag: 'rdc_flutter_sdk_2');
    print('[RDC Log] [UserInfoController 2]: ${DateTime.now()}');
    userInfo = await Get.putAsync(() async => await UserInfoController().init(),
        tag: 'rdc_flutter_sdk_2');
    print('[RDC Log] [ConditionController 3]: ${DateTime.now()}');
    var conditionController = await Get.putAsync(
        () async => await ConditionController().init(),
        tag: 'rdc_flutter_sdk_2');
    print('[RDC Log] [DeviceInfoController 4]: ${DateTime.now()}');
    _deviceInfoController = await Get.putAsync(
        () async => await DeviceInfoController().init(),
        tag: 'rdc_flutter_sdk_2');
    print('[RDC Log] [LocationController 5]: ${DateTime.now()}');
    await Get.putAsync(() async => await LocationController().init(),
        tag: 'rdc_flutter_sdk_2');
    print('[RDC Log] [VersionController 6]: ${DateTime.now()}');
    await Get.putAsync(() async => await VersionController().init(),
        tag: 'rdc_flutter_sdk_2');

    print('[RDC Log] [EventController 7]: ${DateTime.now()}');
    _eventController = await Get.putAsync(
        () async => await EventController().init(),
        tag: 'rdc_flutter_sdk_2');

    apiResult = _eventController.apiResult;
    condition = conditionController.condition;
    changeDB = _eventController.changeDB;

    print('[RDC Log] [Finished 8]: ${DateTime.now()}');
    return this;
  }

  @override
  void setStateLifeCycle(AppLifecycleState state) async {
    //print('setStateLifeCycle SDK ${state.toString()}');
    switch (state) {
      case AppLifecycleState.inactive:
        _eventController.sessionTask?.pause();

        SharedPreferences sharedPreference =
            await SharedPreferences.getInstance();
        if (!sharedPreference.containsKey('pauseTime')) {
          await _eventController.onPause();
        }

        break;
      case AppLifecycleState.paused:
        if (_eventController.session == RabbitSession.start) {
          await _eventController.onPause();
        }
        if (config.collectionType == DataCollectionType.rotation.name) {
          _eventController.rotationTask?.pause();
        }
        break;
      case AppLifecycleState.resumed:
        if (_eventController.session == RabbitSession.start) {
          await _eventController.onResume();
        }
        if (config.collectionType == DataCollectionType.rotation.name) {
          _eventController.rotationTask?.start();
        }
        break;
      case AppLifecycleState.detached:
        break;
    }
  }

  /* ------------------- override config ------------------- */
  @override
  void setAPIKey(String key) {
    assert(key.isNotEmpty);
    config.apiKey = key;
  }

  @override
  void setEncryptionKey(String key) {
    config.encryptKey = key;
  }

  @override
  void setDataCollectionType(DataCollectionType type) {
    if (config.collectionType == type.name) return;

    config.collectionType = type.name;
    _eventController.rotationLoop();
  }

  @override
  void setRetentionDays(int date) {
    assert(date >= 0);

    config.retentionDays = date;
  }

  @override
  void setDeleteAfterSent(bool value) {
    config.deleteAfterSent = value;
  }

  @override
  void setOfflineMode(bool value) {
    config.offlineMode = value;
  }

  @override
  void setRetryOnFailed(int time) {
    assert(time >= 1);
    config.retryTime = time;
  }

  @override
  void setRotationTime(int min) {
    assert(min >= 1);
    if (config.rotationTime == min) return;

    config.rotationTime = min;
    _eventController.rotationLoop();
  }

  @override
  void setTriggerCondition(
      {int cpu = 0, int memory = 0, int batteryLevel = 0}) {
    assert(cpu >= 0);
    assert(memory >= 0);
    assert(batteryLevel >= 0);
    config.cpu = cpu;
    config.memory = memory;
    config.batteryLevel = batteryLevel;
  }

  @override
  void setConsent(bool? value) {
    config.consent = value;
  }

  @override
  void setPermissionMaid(bool value) {
    if (GetPlatform.isIOS) {
      var version = _deviceInfoController.osVersion;
      var versionNum = int.parse(version.split('.').first);
      if (versionNum > 13) {
        return;
      }
    }
    config.permissionMaid = value;
  }

  @override
  void setSessionExpire(int min) {
    assert(min >= 1);
    config.sessionExpire = min;
  }

  @override
  void setSessionAutoExpire(int min) {
    assert(min >= 1);
    config.sessionAutoExpire = min;
  }

  @override
  void setStorageCondition({required int min, required int max}) {
    config.storageMin = min;
    config.storageMax = max;
  }

  /* ------------------- override config ------------------- */

  /* ------------------- set user info  ------------------- */
  @override
  Future<void> setMemberId(String memberId) async {
    userInfo.memberId = memberId;
    await userInfo.save();
  }

  @override
  Future<void> setSsoId(String ssoId) async {
    userInfo.ssoId = ssoId;
    await userInfo.save();
  }
  /* ------------------- set user info  ------------------- */

  /* ------------------- event method  ------------------- */
  @override
  Future<void> addEvent(
      {required String eventName,
      required String eventValue,
      Map<String, String>? dataLayer}) async {
    return await _eventController.addEvent(
        eventName: eventName, eventValue: eventValue, dataLayer: dataLayer);
  }

  @override
  Future<List<RabbitRequest>> getEvents() async {
    return await _eventController.getEvent();
  }

  @override
  Future<void> uploadEvent() async {
    return await _eventController.uploadEvent();
  }

  @override
  Future<void> clearCache() async {
    return await _eventController.clearCache();
  }

  @override
  Future<void> clearAll() async {
    return await _eventController.clearAll();
  }

  @override
  Future<void> sessionStart() async {
    await _eventController.sessionStart();
  }

  @override
  Future<void> sessionEnd() async {
    await _eventController.sessionEnd();
  }

  @override
  Future<void> setDebugMode(bool value) async {
    config.debugMode = value;

    if (value) {
      baseURL = baseStaging;
    } else {
      baseURL = baseProd;
    }

    await _eventController.onChangeDebugMode();
  }

  /* ------------------- event method  ------------------- */
}
