import 'package:get/get.dart';
import 'package:rdc_flutter_sdk_2/src/controllers/rabbit_config_controller.dart';
import 'package:rdc_flutter_sdk_2/src/database/db_interface.dart';
import 'package:rdc_flutter_sdk_2/src/database/hive_db.dart';
import 'package:rdc_flutter_sdk_2/src/model/base_model.dart';

// T generic class, H hive object
class BaseRepo<T extends BaseModel, H extends HiveBaseModel<T>> {
  final RabbitConfigController config = Get.find(tag: 'rdc_flutter_sdk_2');
  late HiveDBInterface hiveDB;
  String boxName = '';

  Future<void> initBase(String name) async {
    boxName = name;
    hiveDB = await HiveDBImpl().initHive(
        encryptKey: config.encryptKey.isEmpty ? null : config.encryptKey);
  }

  Future<List<T>> getAll() async {
    var db = await hiveDB.getAll<H>(boxName);
    List<T> results = [];
    for (var e in db.entries) {
      results.add(e.value.mapFromHive(e.key));
    }
    return results;
  }

  Future<T?> get(int id) async {
    var db = await hiveDB.get<H>(boxName, id);
    return db.isEmpty ? null : db[id]?.mapFromHive(id);
  }

  Future<void> insert(T data) async {
    return await hiveDB.insert<H>(boxName, data.mapToHive());
  }

  Future<void> insertAll(List<T> data) async {
    return await hiveDB.insertAll<H>(
        boxName, data.map((e) => e.mapToHive() as H).toList());
  }

  Future<void> update(int id, T data) async {
    return await hiveDB.upsert<H>(boxName, id, data.mapToHive());
  }

  Future<void> delete(int id) async {
    await hiveDB.delete<H>(boxName, id);
  }

  Future<void> deleteAllKeys(List<int> ids) async {
    await hiveDB.deleteKeys<H>(boxName, ids);
  }

  Future<void> deleteAll() async {
    await hiveDB.deleteAll<H>(boxName);
  }

  Future<T?> first() async {
    try {
      var data = await getAll();
      if (data.isEmpty) return null;
      return data.first;
    } catch (e) {
      //print(e.toString());
    }
    return null;
  }

  Future<T?> last() async {
    try {
      var data = await getAll();
      if (data.isEmpty) return null;
      return data.last;
    } catch (e) {
      //print(e.toString());
    }
    return null;
  }

  Future<int?> lastId() async {
    try {
      var data = await getAll();
      if (data.isEmpty) return null;
      return data.last.id;
    } catch (e) {
      //print(e.toString());
    }
    return null;
  }
}
